/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.component;

import java.io.Serializable;

import org.dussan.vaadin.dtable.client.state.TableState;
import org.dussan.vaadin.dtable.config.TableConfig;
import org.dussan.vaadin.dtable.config.TableExporterConfig;
import org.dussan.vaadin.dtable.helper.ManifestHelper;
import org.dussan.vaadin.dtable.properties.TableProperties;
import org.dussan.vaadin.dtable.query.TableQuery;
import org.dussan.vaadin.dtable.translations.Translation;

import com.vaadin.ui.AbstractComponent;

public class DTableComponent
    extends AbstractComponent
    implements Serializable
{

    private static final long serialVersionUID = -1025248388510196740L;

    protected static final String ARRAY_DELIMITER = "___:_:___";

    protected static final String ARRAY_FIELDS_DELIMITER = "___::___";

    private static final String GIT_VERSION = "Git-Version";

    private static final String GWT_QUERY_VERSION = "GwtQuery-Version";

    private static final String VERSION = "Implementation-Version";

    private TableConfig tableConfig = new TableConfig();

    private TableExporterConfig tableExporterConfig = new TableExporterConfig();

    private TableProperties tableProperties = new TableProperties();

    private TableQuery tableQuery = new TableQuery();

    private Translation translation = new Translation();

    public DTableComponent()
    {
        // Auto-generated constructor stub
    }

    protected static String toStringArrayString( String[] values )
    {
        if ( values != null && 0 < values.length )
        {
            StringBuilder builder = new StringBuilder();
            for ( String value : values )
            {
                builder.append( ARRAY_DELIMITER );
                builder.append( value );
            }

            return "[" + builder.substring( ARRAY_DELIMITER.length() ) + "]";
        }
        return null;
    }

    /**
     * Get dTable version.
     * 
     * @return dTable version
     */
    public static String getVersion()
    {
        if ( ManifestHelper.getManifest() != null )
        {
            return ManifestHelper.getManifest()
                                 .getMainAttributes()
                                 .getValue( VERSION );
        }
        return null;
    }

    /**
     * Get dTable git version.
     * 
     * @return dTable git version
     */
    public static String getGitVersion()
    {
        if ( ManifestHelper.getManifest() != null )
        {
            return ManifestHelper.getManifest()
                                 .getMainAttributes()
                                 .getValue( GIT_VERSION );
        }
        return null;
    }

    /**
     * Get GwtQuery version.
     * 
     * @return GwtQuery version
     */
    public static String getGwtQueryVersion()
    {
        if ( ManifestHelper.getManifest() != null )
        {
            return ManifestHelper.getManifest()
                                 .getMainAttributes()
                                 .getValue( GWT_QUERY_VERSION );
        }
        return null;
    }

    @Override
    protected TableState getState()
    {
        return (TableState) super.getState();
    }

    @Override
    protected TableState getState( boolean markAsDirty )
    {
        return (TableState) super.getState( markAsDirty );
    }

    public Translation getTranslation()
    {
        return translation;
    }

    public void setTranslation( Translation translation )
    {
        this.translation = translation != null ? translation : this.translation;
    }

    public TableExporterConfig getTableExporterConfig()
    {
        return tableExporterConfig;
    }

    public void setTableExporterConfig( TableExporterConfig tableExporterConfig )
    {
        this.tableExporterConfig = tableExporterConfig != null ? tableExporterConfig : this.tableExporterConfig;
    }

    public TableConfig getTableConfig()
    {
        return tableConfig;
    }

    public void setTableConfig( TableConfig tableConfig )
    {
        this.tableConfig = tableConfig != null ? tableConfig : this.tableConfig;
    }

    public TableProperties getTableProperties()
    {
        return tableProperties;
    }

    public void setTableProperties( TableProperties tableProperties )
    {
        this.tableProperties = tableProperties != null ? tableProperties : this.tableProperties;
    }

    public TableQuery getTableQuery()
    {
        return tableQuery;
    }

    public void setTableQuery( TableQuery tableQuery )
    {
        this.tableQuery = tableQuery != null ? tableQuery : this.tableQuery;
    }

}
