/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.test;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.servlet.annotation.WebServlet;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.config.ColumnType;
import org.dussan.vaadin.dtable.config.TableToolsPosition;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;
import org.dussan.vaadin.dtable.properties.filter.Filter;
import org.dussan.vaadin.dtable.properties.order.Order;
import org.dussan.vaadin.dtable.properties.order.OrderType;
import org.dussan.vaadin.dtable.windows.exporter.type.CompressionType;
import org.dussan.vaadin.dtable.windows.exporter.version.MsAccessVersion;
import org.dussan.vaadin.dtable.windows.exporter.version.MsExcelVersion;

import com.vaadin.annotations.Theme;
import com.vaadin.annotations.Title;
import com.vaadin.annotations.VaadinServletConfiguration;
import com.vaadin.annotations.Widgetset;
import com.vaadin.server.VaadinRequest;
import com.vaadin.server.VaadinServlet;
import com.vaadin.ui.UI;
import com.vaadin.ui.VerticalLayout;

@Theme( "dtable-theme" )
@Title( "DTable Add-on Test" )
@Widgetset( "org.dussan.vaadin.dtable.WidgetSet" )
public class DTableUI
    extends UI
{

    private static final long serialVersionUID = 1833595858297782412L;

    private int allRows = 30000;

    private int allColumns = 400;

    private DTable dTable = null;

    public DTableUI()
    {
        dTable = new DTable( "demo" )
        {

            private static final long serialVersionUID = 4671352124365732598L;

            @Override
            public boolean prepareData( Order[] orders, Filter[] filters, Filter[] quickSearch )
            {
                return true;
            }

            @Override
            public long getAllRows( Filter[] filters, Filter[] quickSearch )
            {
                if ( filters != null && 10 < filters.length )
                {
                    return 0;
                }
                return filters == null || filters.length == 0 ? allRows : allRows / ( filters.length + 1 );
            }

            @Override
            public Object[] getRowsData( int startRow, int rows, Order[] orders, Filter[] filters,
                                         Filter[] quickSearch )
            {
                rows = rows == 0 ? allRows : rows;
                List<Object> data = new ArrayList<>();
                for ( int row = startRow; row < ( startRow + rows ) && row < getAllRows( filters, quickSearch ); row++ )
                {
                    for ( int column = 0; column < allColumns; column++ )
                    {
                        if ( column == 0 || column == 3 || column == 9 || column == 10 )
                        {
                            data.add( new Date() );
                        }
                        else if ( column == 5 )
                        {
                            data.add( "Testing, testing, testing, testing, testing, ..." );
                        }
                        else
                        {
                            data.add( (float) row * 100 + (float) column / 100 );
                        }
                    }
                }
                return data.toArray();
            }

            @Override
            public boolean prepareExportedData( String name, Order[] orders, Filter[] filters, Filter[] quickSearch )
            {
                return true;
            }

            @Override
            public long getExportedAllRows( String name, Filter[] orders, Filter[] quickSearch )
            {
                return allRows;
            }

            @Override
            public Object[] getExportedData( String name, int startRow, int rows, Order[] orders, Filter[] filters,
                                             Filter[] quickSearch )
            {
                List<Object> data = new ArrayList<>();
                for ( int row = startRow; row < ( startRow + rows ); row++ )
                {
                    for ( int column = 0; column < allColumns; column++ )
                    {
                        if ( column == 0 || column == 3 || column == 9 || column == 10 )
                        {
                            data.add( new Date() );
                        }
                        else if ( column == 5 )
                        {
                            data.add( "Testing, testing, testing, testing, testing, ..." );
                        }
                        else
                        {
                            data.add( (float) row * 100 + (float) column / 100 );
                        }
                    }
                }
                return data.toArray();
            }

            @Override
            public void exportProcessCompleted()
            {
                // not used
            }
        };
    }

    @Override
    protected void init( VaadinRequest vaadinRequest )
    {
        dTable.getTableConfig()
              .setLocale( new Locale( "sl", "SI" ) );
        dTable.getTableConfig()
              .setAllRowsEnabled( false );
        dTable.getTableConfig()
              .setPageableTableEnabled( true );
        dTable.getTableConfig()
              .setOrderedTableEnabled( true );
        dTable.getTableConfig()
              .setFilteredTableEnabled( true );
        dTable.getTableConfig()
              .setFilterInfoEnabled( true );
        dTable.getTableConfig()
              .setContextMenuForFilteredTableEnabled( true );
        dTable.getTableConfig()
              .setHighlightActiveRowEnabled( true );
        dTable.getTableConfig()
              .setFixedTableHeightEnabled( true );
        dTable.getTableConfig()
              .setOddAndEvenColoringRowsEnabled( true );
        dTable.getTableConfig()
              .setPageSize( 10 );
        dTable.getTableConfig()
              .setPagerPages( 15 );
        dTable.getTableConfig()
              .setColumnDefaultWidth( 50 );
        dTable.getTableConfig()
              .setTableToolsMenuEnabled( true );
        dTable.getTableConfig()
              .setTableToolsMenuAlwaysVisibleEnabled( true );
        dTable.getTableConfig()
              .setTableToolsTrayEnabled( true );

        dTable.getTableExporterConfig()
              .setCompression( CompressionType.ZIP );
        dTable.getTableExporterConfig()
              .setRowIndexEnabled( true );
        dTable.getTableExporterConfig()
              .setFormattedDataEnabled( true );
        dTable.getTableExporterConfig()
              .setExportCsvFileEnabled( true );
        dTable.getTableExporterConfig()
              .setExportMdbFileEnabled( true );
        dTable.getTableExporterConfig()
              .setExportXlsFileEnabled( true );
        dTable.getTableExporterConfig()
              .setTimestampEnabled( true );
        dTable.getTableExporterConfig()
              .setMsAccessVersion( MsAccessVersion.V2010 );
        dTable.getTableExporterConfig()
              .setMsExcelVersion( MsExcelVersion.V2007 );

        dTable.getTableConfig()
              .setTableToolsPosition( TableToolsPosition.BOTTOM_RIGHT );
        dTable.getTableConfig()
              .setSearchEnabled( true );
        dTable.getTableConfig()
              .setSearchHideEnabled( true );
        dTable.getTableConfig()
              .setSearchVisibleAtStartupEnabled( true );
        dTable.getTableConfig()
              .setQuickSearchEnabled( true );
        dTable.getTableConfig()
              .setQuickSearchHideEnabled( true );
        dTable.getTableConfig()
              .setQuickSearchVisibleAtStartupEnabled( true );

        dTable.getTableConfig()
              .setHideUnhideColumnsEnabled( true );
        dTable.getTableConfig()
              .setHideUnhideColumnsToolEnabled( true );
        dTable.getTableConfig()
              .setHideAnyColumnsEnabled( true );
        dTable.getTableConfig()
              .setUnhideAnyColumnsEnabled( true );

        dTable.getTableConfig()
              .setFreezeUnfreezeColumnsEnabled( true );
        dTable.getTableConfig()
              .setFreezeUnfreezeColumnsToolEnabled( true );
        dTable.getTableConfig()
              .setFreezeAnyColumnsEnabled( true );
        dTable.getTableConfig()
              .setUnfreezeAnyColumnsEnabled( true );

        dTable.getTableConfig()
              .setLoadingMessageEnabled( true );
        dTable.getTableConfig()
              .setLoadingMessageOpacity( 0.5f );
        dTable.getTableConfig()
              .setLoadingMessageDelay( 100, 400 );

        dTable.getTableConfig()
              .setFrozenTableEnabled( true );
        dTable.getTableConfig()
              .setFrozenTableVisibleEnabled( true );
        dTable.getTableConfig()
              .setContextMenuForFilteredTableEnabled( true );
        dTable.getTableConfig()
              .setOrderedFrozenTableEnabled( false );
        dTable.getTableConfig()
              .setFilteredFrozenTableEnabled( false );
        dTable.getTableConfig()
              .setHideFrozenTableEnabled( true );
        dTable.getTableConfig()
              .setResizeFrozenTableEnabled( true );
        dTable.getTableConfig()
              .setEmptyFrozenTableEnabled( false );
        dTable.getTableConfig()
              .setTableFrozenMinimalWidth( 200 );

        dTable.getTableConfig()
              .getHeader()
              .setTooltipEnabled( true );
        dTable.getTableConfig()
              .getHeader()
              .setVerticalLinesEnabled( true );
        dTable.getTableConfig()
              .getHeader()
              .setExpandedCellEnabled( true, true );
        dTable.getTableConfig()
              .getHeader()
              .setExpandedCellForActiveColumnEnabled( true );
        dTable.getTableConfig()
              .getHeader()
              .setExpandedFrozenCellEnabled( true, true );
        dTable.getTableConfig()
              .getHeader()
              .setExpandedFrozenCellForActiveColumnEnabled( true );
        dTable.getTableConfig()
              .getHeader()
              .setRowHeight( 24 );
        dTable.getTableConfig()
              .getHeader()
              .setWordWrapEnabled( true );
        dTable.getTableConfig()
              .getHeader()
              .setDefaultColumnAlignment( ColumnAlign.LEFT );

        dTable.getTableConfig()
              .getBody()
              .setVerticalLinesEnabled( true );
        dTable.getTableConfig()
              .getBody()
              .setRowHeight( 30 );
        dTable.getTableConfig()
              .getBody()
              .setLineHeight( 20 );
        dTable.getTableConfig()
              .getBody()
              .setRowAlignment( RowAlign.BOTTOM );
        dTable.getTableConfig()
              .getBody()
              .setWordWrapEnabled( true );
        dTable.getTableConfig()
              .getBody()
              .setExpandedFrozenCellEnabled( true );
        dTable.getTableConfig()
              .getBody()
              .setExpandedCellEnabled( true );
        dTable.getTableConfig()
              .getBody()
              .setDefaultColumnAlignment( ColumnAlign.CENTER );

        dTable.getTableProperties()
              .setFrozenColumns( 1, 2, 9, 11, 12, 13, 14, 15, 16, 17, 18 );
        dTable.getTableConfig()
              .setColumnsFreezeEnabled( false, 4, 5, 7 );
        dTable.getTableConfig()
              .setColumnsUnfreezeEnabled( false, 2, 3 );
        dTable.getTableProperties()
              .setHiddenColumns( 2, 8 );
        dTable.getTableConfig()
              .setColumnHideEnabled( false, 1 );
        dTable.getTableConfig()
              .setColumnUnhideEnabled( false, 2 );
        dTable.getTableProperties()
              .setOrders( OrderType.ASCEDING, 1, 6 );
        dTable.getTableProperties()
              .setOrders( OrderType.DESCEDING, 4, 5 );

        for ( int column = 1; column <= allColumns; column++ )
        {
            if ( column == 1 )
            {
                dTable.setColumn( column - 1, "HEADER " + column, ColumnType.TIMESTAMP );
            }
            else if ( column == 2 )
            {
                dTable.setColumn( column - 1, "HEADER " + column, ColumnType.FLOAT );
            }
            else if ( column == 6 )
            {
                dTable.setColumn( column - 1, "HEADER " + column, ColumnType.TEXT );
                dTable.getTableConfig()
                      .setColumnWidth( column - 1, 200 );
            }
            else if ( column == 4 || column == 11 )
            {
                dTable.setColumn( column - 1, "LONG HEADER " + column, ColumnType.DATE,
                                  new SimpleDateFormat( "dd.MM.yyyy HH:mm:ss" ) );
            }
            else if ( column == 10 )
            {
                dTable.setColumn( column - 1, "HEADER " + column, ColumnType.TIMESTAMP );
            }
            else
            {
                dTable.setColumn( column - 1, "HEADER " + column, ColumnType.FLOAT );
            }

            dTable.getTableProperties()
                  .setHeaderTooltip( column - 1, "HEADER " + column );
        }

        VerticalLayout content = new VerticalLayout();
        content.setSizeFull();
        content.addComponent( dTable );
        setContent( content );
    }

    @WebServlet( urlPatterns = "/*", name = "DTableUIServlet", asyncSupported = true )
    @VaadinServletConfiguration( ui = DTableUI.class, productionMode = false )
    public static class DGridUIServlet
        extends VaadinServlet
    {

        private static final long serialVersionUID = -5596634044315772012L;

    }

}
